//package com.example.testktor
//
//import android.os.Bundle
//import android.util.Log
//import androidx.activity.ComponentActivity
//import androidx.activity.compose.setContent
//import androidx.activity.enableEdgeToEdge
//import androidx.compose.foundation.layout.fillMaxSize
//import androidx.compose.foundation.layout.padding
//import androidx.compose.material3.Scaffold
//import androidx.compose.material3.Text
//import androidx.compose.material3.CircularProgressIndicator
//import androidx.compose.runtime.*
//import androidx.compose.ui.Modifier
//import androidx.compose.ui.tooling.preview.Preview
//import com.example.testktor.ui.theme.TestKtorTheme
//import io.ktor.client.*
//import io.ktor.client.request.*
//import io.ktor.client.engine.android.*
//import io.ktor.client.plugins.contentnegotiation.*
//import io.ktor.client.statement.HttpResponse
//import io.ktor.client.statement.bodyAsText
//import io.ktor.http.ContentType
//import io.ktor.http.contentType
//import kotlinx.coroutines.launch
//import kotlinx.serialization.Serializable
//import io.ktor.serialization.kotlinx.json.*
//import kotlinx.serialization.decodeFromString
//import kotlinx.serialization.json.Json
//
//@Serializable
//data class User(val id: Int, val name: String)
//
//class MainActivity : ComponentActivity() {
//    private val client = HttpClient(Android) {
//        install(ContentNegotiation) {
//            json(Json { prettyPrint = true })
//        }
//    }
//
//    override fun onCreate(savedInstanceState: Bundle?) {
//        super.onCreate(savedInstanceState)
//        enableEdgeToEdge()
//        setContent {
//            TestKtorTheme {
//                Scaffold(modifier = Modifier.fillMaxSize()) { innerPadding ->
//                    UserScreen(modifier = Modifier.padding(innerPadding), client = client)
//                }
//            }
//        }
//    }
//}
//
//@Composable
//fun UserScreen(modifier: Modifier = Modifier, client: HttpClient) {
//    var user by remember { mutableStateOf<User?>(null) }
//    var isLoading by remember { mutableStateOf(true) }
//    var errorMessage by remember { mutableStateOf<String?>(null) }
//
//    val scope = rememberCoroutineScope()
//
//    LaunchedEffect(Unit) {
//        try {
//            val fetchedUser = getUser(client) // Получаем одного пользователя
//            user = fetchedUser
//        } catch (e: Exception) {
//            errorMessage = "Error fetching user: ${e.localizedMessage}"
//            Log.e("UserScreen", "Error: ${e.localizedMessage}", e)
//        }
//        isLoading = false
//    }
//
//    if (isLoading) {
//        CircularProgressIndicator()
//    }
//
//    errorMessage?.let {
//        Text(text = it)
//    }
//
//    user?.let {
//        Text(text = "User: ${it.name}, ID: ${it.id}") // Отображаем имя пользователя
//    } ?: run {
//        Text(text = "No user found or failed to load.")
//    }
//}
//
//suspend fun getUser(client: HttpClient): User {
//    val response: HttpResponse = client.get("http://10.0.2.2:8080/user") {
//        contentType(ContentType.Application.Json)
//    }
//    val responseBody = response.bodyAsText()
//    Log.d("getUser", "Response: $responseBody") // Логируем ответ сервера
//
//    val json = Json { ignoreUnknownKeys = true }
//
//    val buba = json.decodeFromString<User>(responseBody) // Десериализуем в объект User
//    Log.d("json", buba.name)
//
//    return try {
//        json.decodeFromString<User>(responseBody) // Десериализуем в объект User
//    } catch (e: Exception) {
//        Log.e("getUser", "Error deserializing response", e)
//        throw e // Повторно выбрасываем ошибку, чтобы она была обработана в UserScreen
//    }
//}
//
//@Preview(showBackground = true)
//@Composable
//fun UserScreenPreview() {
//    TestKtorTheme {
//        UserScreen(client = HttpClient(Android))
//    }
//}


package com.example.testktor

import android.os.Bundle
import android.util.Log
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.CircularProgressIndicator
import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import androidx.compose.ui.tooling.preview.Preview
import com.example.testktor.ui.theme.TestKtorTheme
import io.ktor.client.*
import io.ktor.client.request.*
import io.ktor.client.engine.android.*
import io.ktor.client.plugins.contentnegotiation.*
import io.ktor.client.statement.HttpResponse
import io.ktor.client.statement.bodyAsText
import io.ktor.http.ContentType
import io.ktor.http.contentType
import kotlinx.coroutines.launch
import kotlinx.serialization.Serializable
import io.ktor.serialization.kotlinx.json.*
import kotlinx.serialization.decodeFromString
import kotlinx.serialization.json.Json

@Serializable
data class User(val id: Int, val name: String)

class MainActivity : ComponentActivity() {
    private val client = HttpClient(Android) {
        install(ContentNegotiation) {
            json(Json { prettyPrint = true })
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        enableEdgeToEdge()
        setContent {
            TestKtorTheme {
                Scaffold(modifier = Modifier.fillMaxSize()) { innerPadding ->
                    UserScreen(modifier = Modifier.padding(innerPadding), client = client)
                }
            }
        }
    }
}

@Composable
fun UserScreen(modifier: Modifier = Modifier, client: HttpClient) {
    var users by remember { mutableStateOf<List<User>?>(null) }
    var isLoading by remember { mutableStateOf(true) }
    var errorMessage by remember { mutableStateOf<String?>(null) }

    val scope = rememberCoroutineScope()

    LaunchedEffect(Unit) {
        try {
            val fetchedUsers = getUsers(client)
            users = fetchedUsers
        } catch (e: Exception) {
            errorMessage = "Error fetching users: ${e.localizedMessage}"
            Log.e("UserScreen", "Error: ${e.localizedMessage}", e)
        }
        isLoading = false
    }

    if (isLoading) {
        CircularProgressIndicator()
    }

    errorMessage?.let {
        Text(text = it)
    }

    users?.let {
        it.forEach { user ->
            Text(text = "User: ${user.name}, ID: ${user.id}")
        }
    } ?: run {
        Text(text = "No users found or failed to load.")
    }
}

suspend fun getUsers(client: HttpClient): List<User> {
    val response: HttpResponse = client.get("http://212.67.10.54:8080/users") {
        contentType(ContentType.Application.Json)
    }
    val responseBody = response.bodyAsText()
    Log.d("getUsers", "Response: $responseBody") // Логируем ответ сервера

    val json = Json { ignoreUnknownKeys = true }

    return try {
        json.decodeFromString<List<User>>(responseBody) // Десериализуем в List<User>
    } catch (e: Exception) {
        Log.e("getUsers", "Error deserializing response", e)
        throw e // Повторно выбрасываем ошибку, чтобы она была обработана в UserScreen
    }
}

@Preview(showBackground = true)
@Composable
fun UserScreenPreview() {
    TestKtorTheme {
        UserScreen(client = HttpClient(Android))
    }
}